@@ -1,3 +1,5 @@
+import datetime
+
 from dateutil.parser import isoparse
 from typing import Iterable
 from coinmetrics._typing import DataFrameType, List, Any, Optional
@@ -6,15 +8,21 @@ from logging import getLogger
 logger = getLogger("cm_client")
 
 try:
-    import pandas as pd  # type: ignore
+    import pandas as pd
 except ImportError:
-    pd = None
     logger.warning(
-        "Pandas export is unavailable. Install pandas to unlock dataframe functions."
+        "Pandas is unavailable. Install pandas to unlock dataframe functions."
+    )
+
+try:
+    import polars as pl
+except ImportError:
+    logger.warning(
+        "Polars is unavailable. Install polars to unlock dataframe functions."
     )
 
 
-def _convert_utc(x: Any) -> DataFrameType:
+def _convert_utc(x: Any) -> Optional[datetime.datetime]:
     try:
         return isoparse(x)
     except TypeError:
@@ -32,16 +40,28 @@ def _expand_df(key: str, iterable: Iterable[Any]) -> List[Any]:
 
 
 def convert_catalog_dtypes(df: DataFrameType) -> DataFrameType:
-    df = df.convert_dtypes()
-    columns = df.columns
-    datetime_cols = [c for c in columns if c.endswith("_time") or c == "time"]
-    for col in datetime_cols:
-        df[col] = df[col].apply(_convert_utc)
+    if isinstance(df, pd.DataFrame):
+        df = df.convert_dtypes()
+        columns = df.columns
+        date_cols = {"expiration", "listing"}
+        datetime_cols = [c for c in columns if "time" in c.split("_") or c == "time" or c in date_cols]
+        for col in datetime_cols:
+            df[col] = df[col].apply(_convert_utc)
+
+    elif isinstance(df, pl.DataFrame):
+        df = df.with_columns([pl.col(c).alias(c) for c in df.columns])
+
+        datetime_cols = [c for c in df.columns if
+                         "time" in c.split("_") or c == "time" or c in {"expiration", "listing"}]
+        df = df.with_columns([
+            pl.col(col).cast(pl.Datetime).alias(col)
+            for col in datetime_cols
+        ])
     return df
 
 
 class CatalogAssetsData(List[Any]):
-    def to_dataframe(self, secondary_level: Optional[str] = None) -> DataFrameType:
+    def to_dataframe(self, secondary_level: Optional[str] = None) -> pd.DataFrame:
         """
         Transforms catalog data in list form into a dataframe
 
@@ -135,7 +155,7 @@ class CatalogAssetsData(List[Any]):
 
 
 class CatalogAssetAlertsData(List[Any]):
-    def to_dataframe(self) -> DataFrameType:
+    def to_dataframe(self) -> pd.DataFrame:
         """
         Transforms catalog data in list form into a dataframe
         :return: Catalog Data
@@ -144,7 +164,7 @@ class CatalogAssetAlertsData(List[Any]):
 
 
 class CatalogAssetChainsData(List[Any]):
-    def to_dataframe(self) -> DataFrameType:
+    def to_dataframe(self) -> pd.DataFrame:
         """
         Transforms catalog data in list form into a dataframe
         :return: Catalog Data
@@ -153,7 +173,7 @@ class CatalogAssetChainsData(List[Any]):
 
 
 class CatalogMempoolFeeratesData(List[Any]):
-    def to_dataframe(self) -> DataFrameType:
+    def to_dataframe(self) -> pd.DataFrame:
         """
         Transforms catalog data in list form into a dataframe
         :return: Catalog Data
@@ -162,7 +182,7 @@ class CatalogMempoolFeeratesData(List[Any]):
 
 
 class CatalogMiningPoolTipsData(List[Any]):
-    def to_dataframe(self) -> DataFrameType:
+    def to_dataframe(self) -> pd.DataFrame:
         """
         Transforms catalog data in list form into a dataframe
         :return: Catalog Data
@@ -171,7 +191,7 @@ class CatalogMiningPoolTipsData(List[Any]):
 
 
 class CatalogTransactionTrackerData(List[Any]):
-    def to_dataframe(self) -> DataFrameType:
+    def to_dataframe(self) -> pd.DataFrame:
         """
         Transforms catalog data in list form into a dataframe
         :return: Catalog Data
@@ -180,7 +200,7 @@ class CatalogTransactionTrackerData(List[Any]):
 
 
 class CatalogMarketTradesData(List[Any]):
-    def to_dataframe(self) -> DataFrameType:
+    def to_dataframe(self) -> pd.DataFrame:
         """
         Transforms catalog data in list form into a dataframe
         :return: Catalog Data
@@ -189,7 +209,7 @@ class CatalogMarketTradesData(List[Any]):
 
 
 class CatalogMarketOrderbooksData(List[Any]):
-    def to_dataframe(self, secondary_level: Optional[str] = None) -> DataFrameType:
+    def to_dataframe(self, secondary_level: Optional[str] = None) -> pd.DataFrame:
         """
         Transforms catalog data in list form into a dataframe
         :return: Catalog Data
@@ -219,7 +239,7 @@ class CatalogMarketOrderbooksData(List[Any]):
 
 
 class CatalogAssetPairsData(List[Any]):
-    def to_dataframe(self) -> DataFrameType:
+    def to_dataframe(self) -> pd.DataFrame:
         """
         Transforms catalog data in list form into a dataframe
         :return: Catalog Data
@@ -252,7 +272,7 @@ class CatalogAssetPairsData(List[Any]):
 
 
 class CatalogExchangesData(List[Any]):
-    def to_dataframe(self, secondary_level: Optional[str] = None) -> DataFrameType:
+    def to_dataframe(self, secondary_level: Optional[str] = None) -> pd.DataFrame:
         """
         Transforms catalog data in list form into a dataframe
 
@@ -302,7 +322,7 @@ class CatalogExchangesData(List[Any]):
 
 
 class CatalogExchangeAssetsData(List[Any]):
-    def to_dataframe(self) -> DataFrameType:
+    def to_dataframe(self) -> pd.DataFrame:
         """
         Transforms catalog data in list form into a dataframe
         :return: Catalog Data
@@ -340,7 +360,7 @@ class CatalogIndexesData(List[Any]):
     :return: Catalog Data
     """
 
-    def to_dataframe(self) -> DataFrameType:
+    def to_dataframe(self) -> pd.DataFrame:
         df_indexes = pd.DataFrame(self)
         df_indexes = (
             df_indexes.explode("frequencies")
@@ -367,7 +387,7 @@ class CatalogInstitutionsData(List[Any]):
     :return: Catalog Data
     """
 
-    def to_dataframe(self) -> DataFrameType:
+    def to_dataframe(self) -> pd.DataFrame:
         df_institutions = pd.DataFrame(self)
         df_institutions = (
             df_institutions.explode("metrics")
@@ -401,7 +421,7 @@ class CatalogMarketsData(List[Any]):
     :return: Catalog Data
     """
 
-    def to_dataframe(self) -> DataFrameType:
+    def to_dataframe(self) -> pd.DataFrame:
         df_markets = pd.DataFrame(self)
         metadata = ["trades", "funding_rates", "openinterest", "liquidations"]
         for col in metadata:
@@ -423,7 +443,7 @@ class CatalogMetricsData(List[Any]):
     :return: Catalog Data
     """
 
-    def to_dataframe(self) -> DataFrameType:
+    def to_dataframe(self) -> pd.DataFrame:
         df_catalog_metrics = (
             pd.DataFrame(self)
             .explode("frequencies")
@@ -446,7 +466,7 @@ class CatalogExchangeAssetMetricsData(List[Any]):
     :return: Catalog Data
     """
 
-    def to_dataframe(self) -> DataFrameType:
+    def to_dataframe(self) -> pd.DataFrame:
         df_catalog_metrics = (
             pd.DataFrame(self)
             .explode("frequencies")
@@ -473,7 +493,7 @@ class CatalogPairMetricsData(List[Any]):
     :return: Catalog Data
     """
 
-    def to_dataframe(self) -> DataFrameType:
+    def to_dataframe(self) -> pd.DataFrame:
         df_catalog_metrics = (
             pd.DataFrame(self)
             .explode("frequencies")
@@ -496,7 +516,7 @@ class CatalogInstitutionMetricsData(List[Any]):
     :return: Catalog Data
     """
 
-    def to_dataframe(self) -> DataFrameType:
+    def to_dataframe(self) -> pd.DataFrame:
         df_catalog_metrics = (
             pd.DataFrame(self)
             .explode("frequencies")
@@ -523,17 +543,31 @@ class CatalogMarketMetricsData(List[Any]):
     :return: Catalog Data
     """
 
-    def to_dataframe(self) -> DataFrameType:
+    def to_dataframe(self) -> pd.DataFrame:
         df_catalog_market_metrics = pd.DataFrame(self)
-        df_catalog_market_metrics = df_catalog_market_metrics.explode("metrics")
-        df_metrics = pd.json_normalize(df_catalog_market_metrics['metrics'])
-        df_catalog_market_metrics = df_catalog_market_metrics.join(df_metrics)
-        df_catalog_market_metrics = df_catalog_market_metrics.explode("frequencies")
-        df_frequencies = pd.json_normalize(df_catalog_market_metrics['frequencies'])
-        df_catalog_market_metrics = df_catalog_market_metrics.join(df_frequencies)
-        df_catalog_market_metrics = df_catalog_market_metrics.drop(["metrics", "frequencies"], axis=1)
-        df_catalog_market_metrics = df_catalog_market_metrics.reset_index(drop=True)
-
+        df_catalog_market_metrics = (
+            df_catalog_market_metrics.explode("metrics")
+            .assign(metric=lambda df: _expand_df(key="metric", iterable=df.metrics))
+            .assign(
+                frequencies=lambda df: _expand_df(
+                    key="frequencies", iterable=df.metrics
+                )
+            )
+            .explode("frequencies")
+            .assign(
+                frequency=lambda df: _expand_df(
+                    key="frequency", iterable=df.frequencies
+                )
+            )
+            .assign(
+                min_time=lambda df: _expand_df(key="min_time", iterable=df.frequencies)
+            )
+            .assign(
+                max_time=lambda df: _expand_df(key="max_time", iterable=df.frequencies)
+            )
+            .reset_index(drop=True)
+            .drop(["metrics", "frequencies"], axis=1)
+        )
         return convert_catalog_dtypes(df_catalog_market_metrics)
 
 
@@ -543,7 +577,7 @@ class CatalogMarketCandlesData(List[Any]):
     :return: Catalog Data
     """
 
-    def to_dataframe(self) -> DataFrameType:
+    def to_dataframe(self) -> pd.DataFrame:
         df_catalog_market_candles = pd.DataFrame(self)
         df_catalog_market_candles = (
             df_catalog_market_candles.explode("frequencies")
@@ -570,7 +604,7 @@ class CatalogAssetPairCandlesData(List[Any]):
     :return: Catalog Data
     """
 
-    def to_dataframe(self) -> DataFrameType:
+    def to_dataframe(self) -> pd.DataFrame:
         df_catalog_asset_candles = pd.DataFrame(self)
         df_catalog_asset_candles = (
             df_catalog_asset_candles.explode("frequencies")
@@ -592,7 +626,7 @@ class CatalogAssetPairCandlesData(List[Any]):
 
 
 class CatalogMarketContractPrices(List[Any]):
-    def to_dataframe(self) -> DataFrameType:
+    def to_dataframe(self) -> pd.DataFrame:
         """
         Transforms catalog data in list form into a dataframe
         :return: Catalog Data
@@ -601,7 +635,7 @@ class CatalogMarketContractPrices(List[Any]):
 
 
 class CatalogMarketImpliedVolatility(List[Any]):
-    def to_dataframe(self) -> DataFrameType:
+    def to_dataframe(self) -> pd.DataFrame:
         """
         Transforms catalog data in list form into a dataframe
         :return: Catalog Data
