@@ -36,8 +36,11 @@ class TransactionTrackerData(CoinMetricsAPIModel):
     status_update_time: str
     status_updates: List[str]
     details: str
+    height: str = field(default="0")
     mempool_approx_queue_position: str = field(default="None")
     next_block_approx_settlement_probability_pct: str = field(default="0")
     block_hash: str = field(default="None")
     geo: str = field(default="None")
     replacement_for_txid: str = field(default="None")
+    inputs: List[str] = field(default_factory=list)
+    outputs: List[str] = field(default_factory=list)
